<?php
/**
 * The client class for patch.
 *
 * @package Woodmart
 */

namespace XTS\Modules\Patcher;

use XTS\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Direct access not allowed.
}

/**
 * The client class for patch.
 */
class Client extends Singleton {
	/**
	 * The uri of the patches remote server.
	 *
	 * @var string
	 */
	public $remote_patches_uri = 'https://xtemos.com/wp-json/xts/v1/patches_maps/';

	/**
	 * Version site.
	 *
	 * @var string
	 */
	public $theme_version;

	/**
	 * Process notices.
	 *
	 * @var array
	 */
	public $notices = array();

	/**
	 * Register hooks and load base data.
	 */
	public function init() {
		$this->theme_version = woodmart_get_theme_info( 'Version' );
	}

	/**
	 * Interface in admin panel.
	 */
	public function interface() {
		wp_enqueue_script( 'woodmart-patcher-scripts', WOODMART_ASSETS . '/js/patcher.js', array(), woodmart_get_theme_info( 'Version' ), true );

		$patches         = $this->get_patches_maps_from_server();
		$patch_installed = get_option( 'woodmart_successfully_installed_patches' );

		?>
		<div class="xts-notices-wrapper xts-patches-notice"><?php $this->print_notices(); // Must be in one line. ?></div>

		<?php if ( $patches ) : ?>
			<div class="xts-patches-wrapper">

				<div class="xts-patch-item xts-patch-title-wrapper">
					<div class="xts-patch-id">
						<?php esc_html_e( 'Patch ID', 'woodmart' ); ?>
					</div>
					<div class="xts-patch-description">
						<?php esc_html_e( 'Description', 'woodmart' ); ?>
					</div>
					<div class="xts-patch-date">
						<?php esc_html_e( 'Date', 'woodmart' ); ?>
					</div>
					<div class="xts-patch-button-wrapper"></div>
				</div>

				<?php foreach ( $patches as $patch_id => $patcher ) : ?>
					<?php $classes = isset( $patch_installed[ $this->theme_version ][ $patch_id ] ) ? ' xts-applyed' : ''; ?>
					<div class="xts-patch-item<?php echo esc_attr( $classes ); ?>">
						<div class="xts-patch-id">
							<?php echo esc_html( $patch_id ); ?>
						</div>
						<div class="xts-patch-description">
							<?php echo apply_filters( 'the_content', $patcher['description'] ); //phpcs:ignore ?>
						</div>
						<div class="xts-patch-date">
							<?php echo esc_html( $patcher['date'] ); ?>
						</div>
						<div class="xts-patch-button-wrapper">
							<?php if ( ! $this->check_filesystem_api() ) : ?>
								<a href="<?php echo esc_url( $patcher['patch_link'] ); ?>" class="xts-btn xts-btn-primary">
									<?php esc_html_e( 'Download', 'woodmart' ); ?>
								</a>
							<?php else : ?>
								<a href="#" class="xts-btn xts-btn-primary xts-patch-apply" data-patches-map='<?php echo wp_json_encode( $patcher['files'] ); ?>' data-id="<?php echo esc_html( $patch_id ); ?>">
									<?php esc_html_e( 'Apply', 'woodmart' ); ?>
								</a>
								<span class="xts-patch-label-applyed">
									<?php esc_html_e( 'Applyed', 'woodmart' ); ?>
								</span>
							<?php endif; ?>
						</div>
					</div>
				<?php endforeach; ?>
			</div>
		<?php endif; ?>
		<?php
	}

	/**
	 * Print notices.
	 */
	public function print_notices() {
		if ( ! $this->check_filesystem_api() ) {
			$this->notices['warning'] = esc_html__( 'Direct access to theme file is not allowed on your server. You need to download and replace the files manually.', 'woodmart' );
		}

		if ( ! $this->notices ) {
			return;
		}

		foreach ( $this->notices as $type => $notice ) {
			$this->print_notice( $notice, $type );
		}
	}

	/**
	 * Print notice.
	 *
	 * @param string $message Message.
	 * @param string $type    Type.
	 */
	private function print_notice( $message, $type = 'warning' ) {
		?>
		<div class="xts-notice xts-<?php echo esc_attr( $type ); ?>">
			<?php echo wp_kses( $message, woodmart_get_allowed_html() ); ?>
		</div>
		<?php
	}

	/**
	 * Queries the patches server for a list of patches.
	 *
	 * @return array
	 */
	public function get_patches_maps_from_server() {
		$url = add_query_arg(
			array(
				'version' => $this->theme_version,
			),
			$this->remote_patches_uri
		);

		$response = wp_remote_get( $url );

		if ( ! isset( $response['body'] ) ) {
			$this->notices['error'] = $response['response']['code'] . ': ' . $response['response']['message'];
			return array();
		}

		$response_body = json_decode( $response['body'], true );

		if ( isset( $response_body['code'] ) && isset( $response_body['message'] ) ) {
			$this->notices['error'] = $response_body['message'];
			return array();
		}

		if ( isset( $response_body['type'] ) && isset( $response_body['message'] ) ) {
			$this->notices[ $response_body['type'] ] = $response_body['message'];
			return array();
		}

		update_option( 'woodmart_patches_map', $response_body );

		return $response_body;
	}

	/**
	 * Check filesystem API.
	 *
	 * @return bool
	 */
	public function check_filesystem_api() {
		global $wp_filesystem;

		if ( function_exists( 'WP_Filesystem' ) ) {
			WP_Filesystem();
		}

		return 'direct' === $wp_filesystem->method;
	}
}

Client::get_instance();
